/**
  @file datetime.h
  
  @author Parallax Inc.
  
  @brief This library provides convenient functions for a 
  variety of timekeeping operations.  Incorporates a manual
  Spin to C translation of Bob Belleville's date_time_ts.spin 
  for timekeeping.
 
  @version 0.5 

  @copyright
  Copyright (c) Parallax Inc 2015. All rights MIT licensed;
                see end of file.
*/  


#ifndef DATETIME_H
#define DATETIME_H

#if defined(__cplusplus)
extern "C" {
#endif

#ifndef SECONDS
/**
 * @brief Constant 1 for 1 second.  
 */
#define SECONDS 1
#endif

#ifndef MINUTES
/**
 * @brief Constant 60 for seconds in a minute.  
 */
#define MINUTES 60
#endif

#ifndef HOURS
/**
 * @brief Constant 3600 for seconds in an hour.  
 */
#define HOURS   60 * MINUTES
#endif

#ifndef DAYS
/**
 * @brief Constant 86400 for seconds in a day.  
 */
#define DAYS    24 * HOURS
#endif

typedef struct datetime_st ///< datetime_st Structure containing y, mo, h, m, s, and etv elements.
{ 
  volatile int y; ///< Year          
  volatile int mo; ///< Month   
  volatile int d; ///< Day   
  volatile int h; ///< Hour   
  volatile int m; ///< Minute   
  volatile int s; ///< Second   
  volatile int etv; ///< epoch time value (seonds from epoch)  
} datetime; ///< datetime Type definition of datetime_st structure. 

/**
 * @brief Start a date/time second counting process in another cog.
 * Example: datetime dts = {2015, 9, 25, 8, 11, 04}; int cogid = 
 * dt_start(&dts);
 * 
 * @param *dts Address of a datetime type, preferably pre-set 
 * before the call.  
 * 
 * @returns cog+1 if successful, 0 if no cogs, -(cog+1) if no locks.
 */
int dt_start(datetime *dts);

/**
 * @brief Stop a date/time second counting process and recover the 
 * cog and lock.
 */
void dt_stop();

/**
 * @brief set a datetime type.  Example datetime mydt; dt_set(&mydt,
 * 2015, 9, 25, 8, 13, 51);  You can also use this to change the 
 * the datetime type that was used in dt_start to "set" the second 
 * counter that auto-increments.
 * 
 * @param *dt Address of datetime type to be set.
 * 
 * @param y Year.
 * 
 * @param mo Month.
 * 
 * @param d Day.
 * 
 * @param h Hour.
 * 
 * @param m Minute.
 * 
 * @param s Second.
 * 
 * @returns etv - the epoch time value, which is a number of seconds
 * from an arbitrarily selected beginning time. 
 */
int dt_set(datetime *dt, int y, int mo, int d, int h, int m, int s);

/**
 * @brief Copy contents of one time type to another.  Example: 
 * datetime a = {2015, 9, 25, 8, 24, 45}; datetime b = {2015, 
 * 9, 25, 8, 24, 35}; dt_copy(&a, &b);  Regardless of what it 
 * stored previously, the a time will contain the b time's contents
 * after the call.
 * 
 * @param *dtDest Address of destination time receives dtSrc values.
 * 
 * @param *dtSrc Address of source time copied to dtDest.
 */
void dt_copy(datetime *dtDest, datetime *dtSrc);

/**
 * @brief Compare etv second values of two datetime types.
 * Example: datetime a = {2015, 9, 25, 8, 24, 45}; datetime
 * b = {2015, 9, 25, 8, 24, 35}; dt_cmpSec(&a, &b);  The result
 * will be 10.  Swap arguments, and the result will be -10.
 * Useful for setting alarms and comparing with system time.
 * 
 * @param *dtTarget Address of target time.  The number
 * of seconds corresponding from this date/time that the dtCmpVal 
 * second value is subracted from.
 * 
 * @param *dtCmpVal The number seconds corresponding to this 
 * date/time are subtracted from the target value.
 *
 * @returns The number of seconds difference: dtTarget - dtCmpVal.
 */
int dt_cmpSec(datetime *dtTarget, datetime *dtCmpVal);

/**
 * @brief Add a number of seconds to a given datetime type.  Example:
 * datetime a = {2001, 1, 1, 0, 0, 0}; dt_addSec(&a, DAYS*1+HOURS*3
 * +MINUTES*4+SECONDS*5); Result: a == {2001, 1, 2, 3, 4, 5} 
 * 
 * @param *dt The daytime type to be modified.
 *
 * @param seconds The number of seconds to add.  May be multiplied by
 * convenient values like DAYS, HOURS, MINUTES, or SECONDS.
 */
void dt_addSec(datetime *dt, int seconds);

#ifndef DOXYGEN_SHOULD_SKIP_THIS

  #ifndef _eunix
  #define _eunix 2440588
  #endif
  
  #ifndef _edos
  #define _edos 2444240
  #endif
  
  #ifndef _eunix
  #define _eunix 2440588
  #endif
  
  #ifndef _eprop
  #define _eprop 2451545
  #endif
  
  #ifndef _epoch
  #define _epoch _eunix
  #endif
  
  int dte_toJD(int y, int m, int d);
  int dte_toSPD(int h, int m, int s);
  int dte_timeETV(int etv);
  int dte_toCal(int jd);
  int dte_dateETV(int etv);
  int dte_toETV(int y, int mo, int d,int h,int m, int s);
  
#endif // DOXYGEN_SHOULD_SKIP_THIS

#if defined(__cplusplus)
}
#endif
/* __cplusplus */  
#endif
/* DATETIME_H */  

/*
  TERMS OF USE: MIT License
 
  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:
 
  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.
 
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

